
/**********************************************************************************
NAME:		crcDumpTableToFile
FUNCTION:	Writes CRC tables to 2 separate files: C and assembly
ARGUMENTS:	char *name - ptr to file name
			void *ptable - ptr to CRC table
			int size - size of table in bytes
			int type - CRC type (16 or 32)
AUTHOR:		Emil Lenchak
DATE:		March, 2004
COPYRIGHT:	Texas Instruments, 2004
TOOLS:		Built with MS Visual C++ 6.0
***********************************************************************************/

#include "stdio.h"
#include "string.h"

#include "..\inc\crc.h"

void crcDumpTableToFile(char *name, void *ptable, int size, int type)
{
	FILE *pfile_c, *pfile_a;
	int i;
	unsigned short *p16 = (unsigned short *)ptable;
	unsigned long *p32 = (unsigned long *)ptable;
	char file_c[128];
	char file_a[128];
	
	strcpy(file_c, name);
	pfile_c = fopen(file_c, "w");

	strcpy(file_a, "_");	// append underscore to file name for assembly file
	strcat(file_a, name);
	pfile_a = fopen(file_a, "w");

	if(pfile_a && pfile_c)
	{
		switch(type)
		{
		case 16:

			for(i = 0 ; i < size-4 ; i+=4)
			{
				fprintf(pfile_c, "  0x%04X, 0x%04X, 0x%04X, 0x%04X,\n", 
					p16[i], p16[i+1], p16[i+2], p16[i+3]);
				fprintf(pfile_a, "  DW 0x%04X, 0x%04X, 0x%04X, 0x%04X\n", 
					p16[i], p16[i+1], p16[i+2], p16[i+3]);
			}

			fprintf(pfile_c, "  0x%04X, 0x%04X, 0x%04X, 0x%04X\n", 
				p16[i], p16[i+1], p16[i+2], p16[i+3]);
			fprintf(pfile_a, "  DW 0x%04X, 0x%04X, 0x%04X, 0x%04X\n", 
				p16[i], p16[i+1], p16[i+2], p16[i+3]);

			break;

		case 32:

			for(i = 0 ; i < size-4 ; i+=4)
			{
				fprintf(pfile_c, "  0x%08X, 0x%08X, 0x%08X, 0x%08X,\n", 
					p32[i], p32[i+1], p32[i+2], p32[i+3]);
				fprintf(pfile_a, "  DL 0x%08X, 0x%08X, 0x%08X, 0x%08X\n", 
					p32[i], p32[i+1], p32[i+2], p32[i+3]);
			}

			fprintf(pfile_c, "  0x%08X, 0x%08X, 0x%08X, 0x%08X\n", 
				p32[i], p32[i+1], p32[i+2], p32[i+3]);
			fprintf(pfile_a, "  DL 0x%08X, 0x%08X, 0x%08X, 0x%08X\n", 
				p32[i], p32[i+1], p32[i+2], p32[i+3]);

			break;

		default:

			printf("ERROR: DATA TYPE NOT SUPORTED\n");
		}
	}
	
	fclose(pfile_c);
	fclose(pfile_a);
}

/*********************************** END ****************************************/
