
/**********************************************************************************
NAME:      main
FUNCTION:  CRC test bench
AUTHOR:    Emil Lenchak
DATE:      March, 2004
COPYRIGHT: Texas Instruments, 2004
TOOLS:     IAR Embedded Workbench V3.20A/MS Visual C++ 6.0
***********************************************************************************/

#ifdef __ICC430__
#include  "msp430x16x.h"
#endif

#ifdef WIN32
#include "conio.h"
#endif

#include "stdio.h"
#include "..\inc\crc.h"

/*************************
	PROTOTYPES
*************************/

#ifdef __cplusplus
extern "C"
{
#endif

void msp430Init(void);

unsigned short crc16MakeBitwise(unsigned short, unsigned short, unsigned char*, unsigned int);
unsigned short crc16MakeBitwise2(unsigned short, unsigned short, unsigned char*, unsigned int);
unsigned short __crc16MakeBitwise(unsigned short, unsigned short, unsigned char*, unsigned int);
unsigned short __crc16rMakeBitwise(unsigned short, unsigned short, unsigned char*, unsigned int);
  
unsigned long crc32MakeBitwise(unsigned long, unsigned long, unsigned char*, unsigned int);
unsigned long crc32MakeBitwise2(unsigned long, unsigned long, unsigned char*, unsigned int);
unsigned long __crc32MakeBitwise(unsigned long, unsigned long, unsigned char*, unsigned int);
unsigned long __crc32rMakeBitwise(unsigned long, unsigned long, unsigned char*, unsigned int);
  
unsigned short crc16MakeTableMethod(unsigned short, TBL_MEM unsigned short*, unsigned char *, unsigned int);
unsigned short crc16rMakeTableMethod(unsigned short, TBL_MEM unsigned short*, unsigned char *, unsigned int);
unsigned short __crc16MakeTableMethod(unsigned short, TBL_MEM unsigned short*, unsigned char*, unsigned int);
//unsigned short __crc16rMakeTableMethod(unsigned short, TBL_MEM unsigned short*, unsigned char*, unsigned int);
  
unsigned long crc32MakeTableMethod(unsigned long, TBL_MEM unsigned long*, unsigned char*, unsigned int);
unsigned long crc32rMakeTableMethod(unsigned long, TBL_MEM unsigned long*, unsigned char*, unsigned int);
unsigned long __crc32MakeTableMethod(unsigned long, TBL_MEM unsigned long*, unsigned char*, unsigned int);
//unsigned long __crc32rMakeTableMethod(unsigned long, TBL_MEM unsigned long*, unsigned char*, unsigned int);

void crc16BuildTable(unsigned short*, unsigned short);
void crc32BuildTable(unsigned long*, unsigned long);
void crcDumpTableToFile(char*, void*, int, int);

unsigned long bitReflect(unsigned long, unsigned int);

#ifdef __cplusplus
}
#endif

/*************************
	EXTERNALS
*************************/

extern TBL_MEM unsigned short crc16Table[CRC_TABLE_SIZE];
extern TBL_MEM unsigned long crc32Table[CRC_TABLE_SIZE];
extern TBL_MEM unsigned short crc16rTable[CRC_TABLE_SIZE];
extern TBL_MEM unsigned long crc32rTable[CRC_TABLE_SIZE];

/*************************
	GLOBALS
*************************/

unsigned char msg[CRC_MSG_MAXSIZE] = CRC_TEST_SEQ;

/*************************
	MAIN
*************************/

int main(int argc, char* argv[])
{
#ifdef WIN32
        int i;
#endif
	unsigned int length;
	unsigned long crc32;
	unsigned short crc16;

#ifdef __ICC430__
	msp430Init();
#endif

	length = sizeof(CRC_TEST_SEQ)-1;

	printf("\nMessage: %s\n\n", msg);

#ifdef WIN32 // dynamically build tables in console app
	crc16BuildTable(crc16Table, CRC16_POLY);
	crc32BuildTable(crc32Table, CRC32_POLY);

	for(i = 0 ; i < CRC_TABLE_SIZE ; i++) // reflect normal tables
	{
		crc16rTable[bitReflect(i, 8)] = (unsigned short)bitReflect(crc16Table[i], 16);
		crc32rTable[bitReflect(i, 8)] = (unsigned long)bitReflect(crc32Table[i], 32);
	}

	// dump tables to files
	crcDumpTableToFile("crc16Table.txt", crc16Table, CRC_TABLE_SIZE, 16);
	crcDumpTableToFile("crc32Table.txt", crc32Table, CRC_TABLE_SIZE, 32);
	crcDumpTableToFile("crc16rTable.txt", crc16rTable, CRC_TABLE_SIZE, 16);
	crcDumpTableToFile("crc32rTable.txt", crc32rTable, CRC_TABLE_SIZE, 32);
#endif  // WIN32

	// calculate CRC brute force alg (optimized implementation for C)
	crc16 = crc16MakeBitwise(CRC16_INIT_REM, CRC16_POLY, msg, length);
	printf("CRC16_bit: 0x%X(N)\n", crc16);

	// calculate CRC brute force alg version #2 (same implementation as assembly routine)
	crc16 = crc16MakeBitwise2(CRC16_INIT_REM, CRC16_POLY, msg, length);
	printf("CRC16_bit: 0x%X(N)\n", crc16);

#ifdef __ICC430__ 
	// calculate CRC brute force alg in assembly
	crc16 = __crc16MakeBitwise(CRC16_INIT_REM, CRC16_POLY, msg, length);
	printf("CRC16_asm: 0x%X(N)\n", crc16);

        // calculate reflected CRC brute force alg in assembly
	crc16 = __crc16rMakeBitwise(CRC16R_INIT_REM, CRC16R_POLY, msg, length);
	printf("CRC16_asm: 0x%X(R)\n", crc16);
#endif	

	// calculate CRC with table method
	crc16 = crc16MakeTableMethod(CRC16_INIT_REM, crc16Table, msg, length);
	printf("CRC16_tab: 0x%X(N)\n", crc16);

	// calculate reflected CRC with table method
	crc16 = crc16rMakeTableMethod(CRC16R_INIT_REM, crc16rTable, msg, length);
	printf("CRC16_tab: 0x%X(R)\n", crc16);

#ifdef __ICC430__ 
	//calculate CRC with table method in assembly
	crc16 = __crc16MakeTableMethod(CRC16_INIT_REM, crc16Table, msg, length);
	printf("CRC16_tasm: 0x%X(N)\n", crc16);
#endif
	// print expected CRCs
	printf("CRC16_exp: 0x%X(N)\n", CRC16_CHECK);	
	printf("CRC16_exp: 0x%X(R)\n\n", CRC16R_CHECK);	

	// calculate CRC brute force alg (optimized implementation for C)
	crc32 = crc32MakeBitwise(CRC32_INIT_REM, CRC32_POLY, msg, length);
	printf("CRC32_bit: 0x%lX(N)\n", crc32);

	// calculate CRC brute force alg version #2 (same implementation as assembly routine)
	crc32 = crc32MakeBitwise2(CRC32_INIT_REM, CRC32_POLY, msg, length);
	printf("CRC32_bit: 0x%lX(N)\n", crc32);

#ifdef __ICC430__ 
	// calculate CRC brute force alg in assembly
	crc32 = __crc32MakeBitwise(CRC32_INIT_REM, CRC32_POLY, msg, length);
	printf("CRC32_asm: 0x%lX(N)\n", crc32);

	// calculate reflected CRC brute force alg in assembly
	crc32 = __crc32rMakeBitwise(CRC32R_INIT_REM, CRC32R_POLY, msg, length);
	printf("CRC32_asm: 0x%lX(R)\n", crc32);	
#endif

	// calculate CRC with table method
	crc32 = crc32MakeTableMethod(CRC32_INIT_REM, crc32Table, msg, length);
	printf("CRC32_tab: 0x%lX(N)\n", crc32);

	// calculate reflected CRC with table method
	crc32 = crc32rMakeTableMethod(CRC32R_INIT_REM, crc32rTable, msg, length);
	printf("CRC32_tab: 0x%lX(R)\n", crc32);

#ifdef __ICC430__
	//calculate CRC with table method in assembly
	crc32 = __crc32MakeTableMethod(CRC32_INIT_REM, crc32Table, msg, length);
	printf("CRC32_tasm: 0x%lX(N)\n", crc32);
#endif
	// print expected CRCs
	printf("CRC32_exp: 0x%lX(N)\n", CRC32_CHECK);	
	printf("CRC32_exp: 0x%lX(R)\n\n", CRC32R_CHECK);	

#ifdef WIN32
	printf("Press any key to continue\n");
	printf("%c\n", getch());
#endif

	return 0;
}

/************************* END MAIN ************************************/
